﻿//Copyright (C) 2007 Christian Scheller

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Domain;
using Storage;

public partial class CreateOrder : System.Web.UI.Page {
    Order Order;
    Customer Customer;

    protected void Page_Load(object sender, EventArgs e) {
        if (!IsPostBack) {
            Order = new Order(Facade.GetNextOrderID());
            OrderNumberLabel.Text = Order.ID.ToString();
            Customer = new Customer(Facade.GetNextCustomerID());
            foreach (Pizza pizza in Facade.GetPizzas().Values) {
                ListItem item = new ListItem(pizza.Name, pizza.ID.ToString());
                ResourceDropDownList.Items.Add(item);
            }
            ViewState.Add("order", Order);
            ViewState.Add("customer", Customer);
        } else {
            Order = (Order) ViewState["order"];
            Customer = (Customer)ViewState["customer"];
            foreach (OrderLine line in Order.DecrementCommitments) {
                AddOrderLineTableRow(line);
            }
        }
    }

    private void AddOrderLineTableRow(OrderLine line) {
        HtmlTableRow row = new HtmlTableRow();
        HtmlTableCell amount = new HtmlTableCell();
        HtmlTableCell resource = new HtmlTableCell();
        amount.InnerText = line.Amount.ToString();
        resource.InnerText = line.Resource.ToString();
        row.Cells.Add(resource);
        row.Cells.Add(amount);
        OrderLineTable.Rows.Add(row);
    }

    protected void AddToOrder_Click(object sender, EventArgs e) {
        OrderLine line = new OrderLine();
        line.Amount = double.Parse(QuantityTextBox.Text);
        line.Resource = Facade.GetPizzas()[int.Parse(ResourceDropDownList.SelectedValue)];
        line.Recipient= Customer;
        Order.DecrementCommitments.Add(line);
        AddOrderLineTableRow(line);
        ViewState.Add("order", Order);
        QuantityTextBox.Text = "1";
        TotalAmountLabel.Text = Order.Total.ToString("#.00");
        TotalAmountPanel.Visible = true;
    }

    protected void Submit_Click(object sender, EventArgs e) {
        Order.AddPayment(Customer, Currency.USD);
        Customer.Name = NameTextBox.Text;
        Customer.Address = AddressTextBox.Text;
        Facade.SaveCustomer(Customer);
        Facade.SaveOrder(Order);
        Response.Redirect("Orders.aspx");
    }

    protected void AlreadyCustomer_Click(object sender, EventArgs e) {
        foreach (Customer customer in Facade.GetCustomers().Values) {
            if (customer.Name == NameTextBox.Text) {
                Customer = customer;
                AddressTextBox.Text = customer.Address;
                ViewState.Add("customer", Customer);
                break;
            }
        }
    }
}
