﻿//Copyright (C) 2007 Christian Scheller

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using Domain;
using REA;

namespace Storage {
    public static class Facade {
        public static Dictionary<int, Pizza> GetPizzas() {
            Dictionary<int, Pizza> pizzas = new Dictionary<int, Pizza>();
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("select id, name, value from pizza", connection);
                SqlDataReader reader = command.ExecuteReader();
                while (reader.Read()) {
                    Pizza pizza = new Pizza();
                    pizza.ID = reader.GetInt32(0);
                    pizza.Name = reader.GetString(1);
                    pizza.Value = reader.GetDouble(2);
                    pizzas.Add(pizza.ID, pizza);
                }
            }
            return pizzas;
        }

        public static Dictionary<int, Customer> GetCustomers() {
            Dictionary<int, Customer> customers = new Dictionary<int, Customer>();
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("select id, name, address from customer", connection);
                SqlDataReader reader = command.ExecuteReader();
                while (reader.Read()) {
                    Customer customer = new Customer(reader.GetInt32(0));
                    customer.Name = reader.GetString(1);
                    customer.Address = reader.GetString(2);
                    customers.Add(customer.ID, customer);
                }
            }
            return customers;
        }

        public static Dictionary<int, Order> GetOrders() {
            Dictionary<int, Order> orders = new Dictionary<int, Order>();
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("select id from [order]", connection);
                SqlDataReader reader = command.ExecuteReader();
                while (reader.Read()) {
                    Order order = new Order(reader.GetInt32(0));
                    orders.Add(order.ID, order);
                }
            }
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("select orderid, resourceid, amount, recipientid, fulfilled from orderline", connection);
                using (SqlDataReader reader = command.ExecuteReader()) {
                    while (reader.Read()) {
                        OrderLine line = new OrderLine();
                        orders[reader.GetInt32(0)].DecrementCommitments.Add(line);
                        line.Resource = Facade.GetPizzas()[reader.GetInt32(1)];
                        line.Amount = reader.GetDouble(2);
                        line.Recipient = GetCustomers()[reader.GetInt32(3)];
                        line.Fulfilled = reader.GetBoolean(4);
                        line.EventCreated += new DecrementEventCreatedHandler(OrderLine_EventCreated);
                    }
                }
                command = new SqlCommand("select orderid, amount, providerid, fulfilled from paymentline", connection);
                using (SqlDataReader reader = command.ExecuteReader()) {
                    while (reader.Read()) {
                        PaymentLine line = new PaymentLine();
                        orders[reader.GetInt32(0)].IncrementCommitments.Add(line);
                        line.Resource = Currency.USD;
                        line.Amount = reader.GetDouble(1);
                        line.Provider = Facade.GetCustomers()[reader.GetInt32(2)];
                        line.Fulfilled = reader.GetBoolean(3);
                        line.EventCreated += new IncrementEventCreatedHandler(PaymentLine_EventCreated);
                    }
                }
            }
            return orders;
        }

        static void OrderLine_EventCreated(DecrementEvent e) {
            SaveDecrementEvent(e);
        }

        static void PaymentLine_EventCreated(IncrementEvent e) {
            SaveIncrementEvent(e);
        }

        public static void SaveIncrementEvent(IncrementEvent e) {
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("insert into incrementevent (amount, providerid, date) values (@amount, @providerid, @date)", connection);
                command.Parameters.AddWithValue("@amount", e.Amount);
                command.Parameters.AddWithValue("@providerid", e.Provider.ID);
                command.Parameters.AddWithValue("@date", e.Date);
                command.ExecuteNonQuery();
            }
        }

        public static void SaveDecrementEvent(DecrementEvent e) {
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("insert into decrementevent (amount, resourceid, recipientid, date) values (@amount, @resourceid, @recipientid, @date)", connection);
                command.Parameters.AddWithValue("@amount", e.Amount);
                command.Parameters.AddWithValue("@resourceid", e.Resource.ID);
                command.Parameters.AddWithValue("@recipientid", e.Recipient.ID);
                command.Parameters.AddWithValue("@date", e.Date);
                command.ExecuteNonQuery();
            }
        }

        public static void SaveOrder(Order order) {
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("select 'x' from [order] where id = @id", connection);
                command.Parameters.AddWithValue("@id", order.ID);
                if (command.ExecuteScalar() == null) {
                    command = new SqlCommand("insert into [order] (id) values (@id)", connection);
                    command.Parameters.AddWithValue("@id", order.ID);
                    command.ExecuteNonQuery();
                }
                foreach (OrderLine line in order.DecrementCommitments) {
                    SaveOrderLine(order, line);
                }
                foreach (PaymentLine line in order.IncrementCommitments) {
                    SavePaymentLine(order, line);
                }
            }
        }

        public static void SaveCustomer(Customer customer) {
            if (!GetCustomers().ContainsKey(customer.ID)) {
                using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                    connection.Open();
                    SqlCommand command = new SqlCommand("insert into customer (id, name, address) values (@id, @name, @address)", connection);
                    command.Parameters.AddWithValue("@id", customer.ID);
                    command.Parameters.AddWithValue("@name", customer.Name);
                    command.Parameters.AddWithValue("@address", customer.Address);
                    command.ExecuteNonQuery();
                }
            }
        }

        public static void SaveOrderLine(Order order, OrderLine orderline) {
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("update orderline set fulfilled = @fulfilled, amount = @amount where orderid = @orderid and resourceid = @resourceid and recipientid = @recipientid", connection);
                command.Parameters.AddWithValue("@fulfilled", orderline.Fulfilled);
                command.Parameters.AddWithValue("@orderid", order.ID);
                command.Parameters.AddWithValue("@resourceid", orderline.Resource.ID);
                command.Parameters.AddWithValue("@amount", orderline.Amount);
                command.Parameters.AddWithValue("@recipientid", orderline.Recipient.ID);
                if (command.ExecuteNonQuery() == 0) {
                    command = new SqlCommand("insert into orderline (orderid, resourceid, amount, recipientid, fulfilled) values (@orderid, @resourceid, @amount, @recipientid, @fulfilled)", connection);
                    command.Parameters.AddWithValue("@fulfilled", orderline.Fulfilled);
                    command.Parameters.AddWithValue("@orderid", order.ID);
                    command.Parameters.AddWithValue("@resourceid", orderline.Resource.ID);
                    command.Parameters.AddWithValue("@amount", orderline.Amount);
                    command.Parameters.AddWithValue("@recipientid", orderline.Recipient.ID);
                    command.ExecuteNonQuery();
                }
            }
        }

        public static void SavePaymentLine(Order order, PaymentLine paymentline) {
            using (SqlConnection connection = new SqlConnection("Data Source=chs-a261\\sqlexpress;Initial Catalog=JoesPizzeria;Integrated Security=True")) {
                connection.Open();
                SqlCommand command = new SqlCommand("update paymentline set fulfilled = @fulfilled, amount = @amount where orderid = @orderid and providerid = @providerid", connection);
                command.Parameters.AddWithValue("@fulfilled", paymentline.Fulfilled);
                command.Parameters.AddWithValue("@orderid", order.ID);
                command.Parameters.AddWithValue("@amount", paymentline.Amount);
                command.Parameters.AddWithValue("@providerid", paymentline.Provider.ID);
                if (command.ExecuteNonQuery() == 0) {
                    command = new SqlCommand("insert into paymentline (orderid, amount, providerid, fulfilled) values (@orderid, @amount, @providerid, @fulfilled)", connection);
                    command.Parameters.AddWithValue("@fulfilled", paymentline.Fulfilled);
                    command.Parameters.AddWithValue("@orderid", order.ID);
                    command.Parameters.AddWithValue("@amount", paymentline.Amount);
                    command.Parameters.AddWithValue("@providerid", paymentline.Provider.ID);
                    command.ExecuteNonQuery();
                }
            }
        }

        public static int GetNextOrderID() {
            int id = 10000;
            foreach (Order item in Facade.GetOrders().Values) {
                id = Math.Max(id, item.ID);
            }
            return ++id;
        }

        public static int GetNextCustomerID() {
            int id = 1000;
            foreach (Customer item in Facade.GetCustomers().Values) {
                id = Math.Max(id, item.ID);
            }
            return ++id;
        }
    }
}
