﻿//Copyright (C) 2007 Christian Scheller

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Storage;
using REA;
using Domain;

namespace TestData {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
        }

        Random rnd = new Random();

        private void button1_Click(object sender, EventArgs e) {
            double amount = 0;
            foreach (DecrementEvent ev in GetDecrementEvents(1000)) {
                amount += ev.Resource.Value * ev.Amount;
                Facade.SaveDecrementEvent(ev);
            }
            foreach (IncrementEvent ev in GetIncrementEvents(amount)) {
                Facade.SaveIncrementEvent(ev);
            }
        }

        public IEnumerable<DecrementEvent> GetDecrementEvents(int count) {
            List<Pizza> pizzas = new List<Pizza>(Facade.GetPizzas().Values);
            List<Customer> customers = new List<Customer>(Facade.GetCustomers().Values);
            for (int i = 0; i < count; ++i) {
                int month = func<int>(new int[] {40,90,130, 157,177,185,192,195,192,200,212,215}, new int[] {1,2,3,4,5,6,7,8,9,10,11,12});
                int day = rnd.Next(1, 28);
                Pizza pizza = func<Pizza>(new int[] {30,40,10,17,1,27,12}, pizzas.ToArray());
                double amount = func<double>(new int[] { 10, 5, 2, 1, 1 }, new double[] { 1, 2, 3, 4, 5 });
                Agent recipient = func<Agent>(new int[] { 5, 7, 6, 3, 5 }, customers.ToArray());
                DecrementEvent e = new DecrementEvent(new DateTime(2005, month, day), pizza, amount, recipient);
                yield return e;
            }
        }

        public IEnumerable<IncrementEvent> GetIncrementEvents(double total) {
            List<Pizza> pizzas = new List<Pizza>(Facade.GetPizzas().Values);
            List<Customer> customers = new List<Customer>(Facade.GetCustomers().Values);
            while (total > 0) {
                int month = func<int>(new int[] { 40, 90, 130, 157, 177, 185, 192, 195, 192, 200, 212, 215 }, new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 });
                int day = rnd.Next(1, 28);
                Pizza pizza = func<Pizza>(new int[] { 30, 40, 10, 17, 1, 27, 12 }, pizzas.ToArray());
                double amount = func<double>(new int[] { 10, 5, 2, 1, 1 }, new double[] { 1, 2, 3, 4, 5 });
                Agent recipient = func<Agent>(new int[] { 5, 7, 6, 3, 5 }, customers.ToArray());
                IncrementEvent e = new IncrementEvent(new DateTime(2005, month, day), Currency.USD, amount * pizza.Value, recipient);
                yield return e;
                total -= pizza.Value * amount;
            }
        }

        public T func<T>(int[] probability, T[] values) {
            int max = 0;
            foreach (int x in probability) {
                max += x;
            }
            int value = rnd.Next(0, max);
            for (int i = 0; i < probability.Length; i++) {
                if (value < probability[i]) {
                    return values[i];
                } else {
                    value -= probability[i];
                }
            }
            throw new ApplicationException("out");
        }
    }
}